%define use_clang 1
%define dist hepx
Summary: An interpreted, interactive, object-oriented programming language
Name: python
Version: 2.7.18
Release: 14%{?dist}4a
Source: http://www.python.org/ftp/python/%{version}/Python-%{version}.tar.gz
Patch1: localhost/%{name}-%{version}-macos13.patch
Patch2: localhost/%{name}-%{version}-openssl1.patch
# info part of this package is under GFDL license
# eigen/nonsymmv.c and eigen/schur.c
# contains rutiens which are part of LAPACK - under BSD style license
License: Python
Group: Development/Languages
Requires: %{name}-libs%{?_isa} = %{version}-%{release}
BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}-root
BuildRequires: pkgconfig

BuildArch: fat

%description
Python is an interpreted, interactive, object-oriented programming
language often compared to Tcl, Perl, Scheme or Java. Python includes
modules, classes, exceptions, very high level dynamic data types and
dynamic typing. Python supports interfaces to many system calls and
libraries, as well as to various windowing systems (X11, Motif, Tk,
Mac and MFC).

Programmers can write new built-in modules for Python in C or C++.
Python can be used as an extension language for applications that need
a programmable interface.

Note that documentation for Python is provided in the python-docs
package.

This package provides the "python" executable; most of the actual
implementation is within the "python-libs" package.

%package libs
Summary: Runtime libraries for Python
Group: Applications/System

%description libs
This package contains runtime libraries for use by Python:
- the libpython dynamic library, for use by applications that embed Python as
a scripting language, and by the main "python" executable
- the Python standard library

%package devel
Summary: The libraries and header files needed for Python development
Group: Development/Libraries
Requires: %{name}%{?_isa} = %{version}-%{release}

%description devel
The Python programming language's interpreter can be extended with
dynamically loaded extensions and can be embedded in other programs.
This package contains the header files and libraries needed to do
these types of tasks.

Install python-devel if you want to develop Python extensions.  The
python package will also need to be installed.  You'll probably also
want to install the python-docs package, which contains Python
documentation.

%prep
%setup -q -c %{name}-%{version}
mv %{name}-%{version} ARM64
pushd ARM64
%patch 1 -p1 -b .macos13
%patch 2 -p1 -b .openssl
cp -a Doc LICENSE README ..
popd
cp -a ARM64 X86_64


%build
pushd ARM64
export CFLAGS="-O3 -arch arm64 -I%{_includedir} -I$(xcrun --show-sdk-path)/usr/include"
export CPPFLAGS="-arch arm64 -I%{_includedir} -I$(xcrun --show-sdk-path)/usr/include"
export CXXFLAGS="$CFLAGS"
export FFLAGS="$CFLAGS"
export FCFLAGS="$CFLAGS"
export LDFLAGS="-arch arm64 -L%{_libdir} -L$(xcrun --show-sdk-path)/usr/lib"
%if %{use_clang}
export CC=clang
export CC_FOR_BUILD=clang
export CXX=clang++
%else
export CC=gcc
export CC_FOR_BUILD=gcc
export CXX=g++
%endif

./configure \
	--prefix=%{_prefix} --enable-shared --with-system-ffi \
	--with-ensurepip=upgrade
make %{?_smp_mflags}
popd

pushd X86_64
export CFLAGS="-O3 -arch x86_64 -I%{_includedir} -I$(xcrun --show-sdk-path)/usr/include"
export CPPFLAGS="-arch x86_64 -I%{_includedir} -I$(xcrun --show-sdk-path)/usr/include"
export CXXFLAGS="$CFLAGS"
export FFLAGS="$CFLAGS"
export FCFLAGS="$CFLAGS"
export LDFLAGS="-arch x86_64 -L%{_libdir} -L$(xcrun --show-sdk-path)/usr/lib"
%if %{use_clang}
export CC=clang
export CC_FOR_BUILD=clang
export CXX=clang++
%else
export CC=gcc
export CC_FOR_BUILD=gcc
export CXX=g++
%endif

arch -arch x86_64 \
./configure \
	--prefix=%{_prefix} --enable-shared --with-system-ffi \
	--with-ensurepip=upgrade
arch -arch x86_64 \
make %{?_smp_mflags}
popd


%install
[ "%{buildroot}" != / ] && rm -rf %{buildroot}
PWD=`pwd`
pushd ARM64
rm -rf ${PWD}-root
mkdir -p ${PWD}-root
make install DESTDIR=${PWD}-root
popd

pushd X86_64
rm -rf ${PWD}-root
mkdir -p ${PWD}-root
make install DESTDIR=${PWD}-root
popd

## Make Universal Binaries
filelist=$(find ./ARM64-root -type f | xargs file | sed -e 's,^\./ARM64-root/,,g' | \
        grep -E \(Mach-O\)\|\(ar\ archive\) |sed -e 's,:.*,,g' -e '/\for\ architecture/d')

for i in $filelist
do
        /usr/bin/lipo -create ARM64-root/$i X86_64-root/$i -output `basename $i` && \
        (cp -f `basename $i` ARM64-root/$i) || :
done

# check header files
for i in `find ARM64-root -name "*.h" -type f`
do
        TARGET=`echo $i | sed -e "s,.*ARM64-root,,"`
        TEMP=`diff -u ARM64-root/$TARGET X86_64-root/$TARGET > /dev/null || echo different`
        if [ -n "$TEMP" ]; then
                mv X86_64-root/$TARGET ARM64-root/${TARGET%.*}-x86_64.h
                mv ARM64-root/$TARGET ARM64-root/${TARGET%.*}-arm64.h
                FILE=${TARGET##*/}
                FILE=${FILE%.*}
cat <<EOF > ARM64-root/$TARGET
#if defined (__arm64__)
#include "${FILE}-arm64.h"
#elif defined( __x86_64__ )
#include "${FILE}-x86_64.h"
#endif
EOF
        fi
done

# install
mkdir -p %{buildroot}
tar cf - -C ARM64-root . | tar xpf - -C %{buildroot}

## remove unuse files
rm -rf $RPM_BUILD_ROOT%{_libdir}/*.la
rm -rf $RPM_BUILD_ROOT%{_datadir}/common-lisp
rm -f $RPM_BUILD_ROOT%{_infodir}/dir


%ifnos darwin
%post -p /sbin/ldconfig

%postun -p /sbin/ldconfig
%endif

%clean
[ "%{buildroot}" != / ] && rm -rf %{buildroot}


%files
%ifnos darwin
%defattr(-,root,root,-)
%else
%defattr(-,root,wheel,-)
%endif
%doc Doc LICENSE README
%{_bindir}/*
%exclude %{_bindir}/*-config
%dir %{_libdir}/python2.7
%{_libdir}/python2.7/*
%{_datadir}/man/man1/*

%files libs
%ifnos darwin
%defattr(-,root,root,-)
%else
%defattr(-,root,wheel,-)
%endif
%doc LICENSE README
%ifnos darwin
%{_libdir}/*so.*
%else
%{_libdir}/*.*.dylib
%endif

%files devel
%ifnos darwin
%defattr(-,root,root,-)
%else
%defattr(-,root,wheel,-)
%endif
%doc LICENSE README
%{_bindir}/*-config
%dir %{_includedir}/python2.7
%{_includedir}/python2.7/*


%changelog
* Fri Dec 13 2024 Keisuke Fujii <keisuke.fujii@kek.jp> - 2.7.18-14hepx4a
- 1st build and release on MacOS 14
- openssl1 now under /opt/openssl/1.11.1q, which requires openssl1 patch
- built with clang and OSXWS 14

* Sun Nov 12 2023 Keisuke Fujii <keisuke.fujii@kek.jp> - 2.7.18-13hepx3a
- 3rd build and release on MacOS 13
- replace arm64 patch by macos13 patch (which is mostly from MacPorts)
- now as fat binaries
- added pip
- built with clang and OSXWS 13

* Sat Nov 11 2023 Keisuke Fujii <keisuke.fujii@kek.jp> - 2.7.18-13hepx2a
- 2nd build and release on MacOS 13
- now as fat binaries
- added pip
- built with clang and OSXWS 13

* Fri Nov 3 2023 Keisuke Fujii <keisuke.fujii@kek.jp> - 2.7.18-13hepx1a
- 1st build and release on MacOS 13
- now as fat binaries
- built with clang and OSXWS 13

* Sun Feb 27 2022 Keisuke Fujii <keisuke.fujii@kek.jp> - 2.7.18-12hepx1b
- 2nd build and release on MacOS X 12
- built with OSXWS 12beta

* Sat Feb 19 2022 Keisuke Fujii <keisuke.fujii@kek.jp> - 2.7.18-12hepx1a
- 1st build and release on MacOS X 12
- built with OSXWS 11
